@@ -25,21 +25,24 @@ import pipes
 import platform
 import zipfile
 import shutil
+import glob
 
 try:  # pragma: no cover (py2 only)
     from ConfigParser import SafeConfigParser as ConfigParser
+    # noinspection PyCompatibility
     from HTMLParser import HTMLParser
     import urllib2
     iteritems = operator.methodcaller('iteritems')
 except ImportError:  # pragma: no cover (py3 only)
     from configparser import ConfigParser
+    # noinspection PyUnresolvedReferences
     from html.parser import HTMLParser
     import urllib.request as urllib2
     iteritems = operator.methodcaller('items')
 
 from pkg_resources import parse_version
 
-nodeenv_version = '1.1.1'
+nodeenv_version = '1.3.3'
 
 join = os.path.join
 abspath = os.path.abspath
@@ -50,6 +53,7 @@ if is_PY3:
     from functools import cmp_to_key
 
 is_WIN = platform.system() == 'Windows'
+is_CYGWIN = platform.system().startswith('CYGWIN')
 
 
 # ---------------------------------------------------------
@@ -86,7 +90,7 @@ class Config(object):
     # Defaults
     node = 'latest'
     npm = 'latest'
-    with_npm = True if is_WIN else False
+    with_npm = True if is_WIN or is_CYGWIN else False
     jobs = '2'
     without_ssl = False
     debug = False
@@ -174,8 +178,8 @@ def create_logger():
     Create logger for diagnostic
     """
     # create logger
-    logger = logging.getLogger("nodeenv")
-    logger.setLevel(logging.INFO)
+    loggr = logging.getLogger("nodeenv")
+    loggr.setLevel(logging.INFO)
 
     # monkey patch
     def emit(self, record):
@@ -196,8 +200,8 @@ def create_logger():
     ch.setFormatter(formatter)
 
     # add ch to logger
-    logger.addHandler(ch)
-    return logger
+    loggr.addHandler(ch)
+    return loggr
 
 
 logger = create_logger()
@@ -339,7 +343,7 @@ def parse_args(check=True):
 
     options, args = parser.parse_args()
     if options.config_file is None:
-        options.config_file = ["./setup.cfg", "~/.nodeenvrc"]
+        options.config_file = ["./tox.ini", "./setup.cfg", "~/.nodeenvrc"]
     elif not options.config_file:
         options.config_file = []
     else:
@@ -376,6 +380,7 @@ def mkdir(path):
         logger.debug(' * Directory %s already exists', path)
 
 
+# noinspection PyArgumentList
 def writefile(dest, content, overwrite=True, append=False):
     """
     Create file and write content in it
@@ -407,11 +412,7 @@ def writefile(dest, content, overwrite=True, append=False):
         if append:
             logger.info(' * Appending data to %s', dest)
             with open(dest, 'ab') as f:
-                if not is_WIN:
-                    f.write(DISABLE_PROMPT.encode('utf-8'))
                 f.write(content)
-                if not is_WIN:
-                    f.write(ENABLE_PROMPT.encode('utf-8'))
             return
 
         logger.info(' * Overwriting %s with new content', dest)
@@ -465,7 +466,13 @@ def callit(cmd, show_stdout=True, in_shell=False,
         line = stdout.readline()
         if not line:
             break
-        line = line.decode('utf-8').rstrip()
+        try:
+            if is_WIN:
+                line = line.decode('mbcs').rstrip()
+            else:
+                line = line.decode('utf8').rstrip()
+        except UnicodeDecodeError:
+            line = line.decode('cp866').rstrip()
         all_output.append(line)
         if show_stdout:
             logger.info(line)
@@ -486,7 +493,7 @@ def get_root_url(version):
     if parse_version(version) > parse_version("0.5.0"):
         return 'https://%s/dist/v%s/' % (src_domain, version)
     else:
-        return 'https://%s/dist/' % (src_domain)
+        return 'https://%s/dist/' % src_domain
 
 
 def get_node_bin_url(version):
@@ -494,16 +501,17 @@ def get_node_bin_url(version):
         'x86':    'x86',  # Windows Vista 32
         'i686':   'x86',
         'x86_64': 'x64',  # Linux Ubuntu 64
+        'amd64':  'x64',  # FreeBSD 64bits
         'AMD64':  'x64',  # Windows Server 2012 R2 (x64)
         'armv6l': 'armv6l',     # arm
         'armv7l': 'armv7l',
-        'aarch64': 'armv64',
+        'aarch64': 'arm64',
     }
     sysinfo = {
         'system': platform.system().lower(),
         'arch': archmap[platform.machine()],
     }
-    if is_WIN:
+    if is_WIN or is_CYGWIN:
         filename = 'win-%(arch)s/node.exe' % sysinfo
     else:
         postfix = '-%(system)s-%(arch)s.tar.gz' % sysinfo
@@ -534,7 +542,7 @@ def download_node_src(node_url, src_dir, opt, prefix):
     dl_contents = io.BytesIO(urlopen(node_url).read())
     logger.info('.', extra=dict(continued=True))
 
-    if is_WIN:
+    if is_WIN or is_CYGWIN:
         writefile(join(src_dir, 'node.exe'), dl_contents.read())
     else:
         with tarfile_open(fileobj=dl_contents) as tarfile_obj:
@@ -559,6 +567,24 @@ def urlopen(url):
 # Virtual environment functions
 
 
+def copytree(src, dst, symlinks=False, ignore=None):
+    for item in os.listdir(src):
+        s = os.path.join(src, item)
+        d = os.path.join(dst, item)
+        if os.path.isdir(s):
+            try:
+                shutil.copytree(s, d, symlinks, ignore)
+            except OSError:
+                copytree(s, d, symlinks, ignore)
+        else:
+            if os.path.islink(s):
+                # copy link only if it not exists. #189
+                if not os.path.islink(d):
+                    os.symlink(os.readlink(s), d)
+            else:
+                shutil.copy2(s, d)
+
+
 def copy_node_from_prebuilt(env_dir, src_dir, node_version):
     """
     Copy prebuilt binaries into environment
@@ -570,9 +596,16 @@ def copy_node_from_prebuilt(env_dir, src_dir, node_version):
         dst_exe = join(env_dir, 'Scripts', 'node.exe')
         mkdir(join(env_dir, 'Scripts'))
         callit(['copy', '/Y', '/L', src_exe, dst_exe], False, True)
+    elif is_CYGWIN:
+        mkdir(join(env_dir, 'bin'))
+        writefile(join(env_dir, 'bin', 'node'), CYGWIN_NODE)
+        src_exe = join(src_dir, 'node.exe')
+        dst_exe = join(env_dir, 'bin', 'node.exe')
+        callit(['cp', '-a', src_exe, dst_exe], True, env_dir)
     else:
-        src_folder = src_dir + to_utf8('/%s-v%s*/*' % (prefix, node_version))
-        callit(['cp', '-a', src_folder, env_dir], True, env_dir)
+        src_folder_tpl = src_dir + to_utf8('/%s-v%s*' % (prefix, node_version))
+        for src_folder in glob.glob(src_folder_tpl):
+            copytree(src_folder, env_dir, True)
     logger.info('.', extra=dict(continued=True))
 
 
@@ -612,9 +645,10 @@ def build_node_from_src(env_dir, src_dir, node_src_dir, opt):
         env['PATH'] = '{}:{}'.format(node_tmpbin_dir,
                                      os.environ.get('PATH', ''))
 
-    conf_cmd = []
-    conf_cmd.append('./configure')
-    conf_cmd.append('--prefix=%s' % pipes.quote(env_dir))
+    conf_cmd = [
+        './configure',
+        '--prefix=%s' % pipes.quote(env_dir)
+    ]
     if opt.without_ssl:
         conf_cmd.append('--without-ssl')
     if opt.debug:
@@ -667,14 +701,13 @@ def install_node(env_dir, src_dir, opt):
     logger.info(' done.')
 
 
-def install_npm(env_dir, src_dir, opt):
+def install_npm(env_dir, _src_dir, opt):
     """
     Download source code for npm, unpack it
     and install it in virtual environment.
     """
     logger.info(' * Install npm.js (%s) ... ' % opt.npm,
                 extra=dict(continued=True))
-    npm_contents = urlopen('https://www.npmjs.org/install.sh').read()
     env = dict(
         os.environ,
         clean='no' if opt.no_npm_clean else 'yes',
@@ -683,8 +716,9 @@ def install_npm(env_dir, src_dir, opt):
     proc = subprocess.Popen(
         (
             'bash', '-c',
-            '. {0} && exec bash'.format(
+            '. {0} && npm install -g npm@{1}'.format(
                 pipes.quote(join(env_dir, 'bin', 'activate')),
+                opt.npm,
             )
         ),
         env=env,
@@ -692,7 +726,7 @@ def install_npm(env_dir, src_dir, opt):
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
     )
-    out, _ = proc.communicate(npm_contents)
+    out, _ = proc.communicate()
     if opt.verbose:
         logger.info(out)
     logger.info('done.')
@@ -705,7 +739,7 @@ def install_npm_win(env_dir, src_dir, opt):
     """
     logger.info(' * Install npm.js (%s) ... ' % opt.npm,
                 extra=dict(continued=True))
-    npm_url = 'https://github.com/npm/npm/archive/%s.zip' % opt.npm
+    npm_url = 'https://github.com/npm/cli/archive/%s.zip' % opt.npm
     npm_contents = io.BytesIO(urlopen(npm_url).read())
 
     bin_path = join(env_dir, 'Scripts')
@@ -723,13 +757,22 @@ def install_npm_win(env_dir, src_dir, opt):
     with zipfile.ZipFile(npm_contents, 'r') as zipf:
         zipf.extractall(src_dir)
 
-    npm_ver = 'npm-%s' % opt.npm
+    npm_ver = 'cli-%s' % opt.npm
     shutil.copytree(join(src_dir, npm_ver), node_modules_path)
     shutil.copy(join(src_dir, npm_ver, 'bin', 'npm.cmd'),
                 join(bin_path, 'npm.cmd'))
     shutil.copy(join(src_dir, npm_ver, 'bin', 'npm-cli.js'),
                 join(bin_path, 'npm-cli.js'))
 
+    if is_CYGWIN:
+        shutil.copy(join(bin_path, 'npm-cli.js'),
+                    join(env_dir, 'bin', 'npm-cli.js'))
+        shutil.copytree(join(bin_path, 'node_modules'),
+                        join(env_dir, 'bin', 'node_modules'))
+        npm_gh_url = 'https://raw.githubusercontent.com/npm/npm'
+        npm_bin_url = '{}/{}/bin/npm'.format(npm_gh_url, opt.npm)
+        writefile(join(env_dir, 'bin', 'npm'), urlopen(npm_bin_url).read())
+
 
 def install_packages(env_dir, opt):
     """
@@ -772,10 +815,16 @@ def install_activate(env_dir, opt):
         shim_node = join(bin_dir, "node.exe")
         shim_nodejs = join(bin_dir, "nodejs.exe")
     else:
-        files = {'activate': ACTIVATE_SH, 'shim': SHIM}
+        files = {
+            'activate': ACTIVATE_SH,
+            'activate.fish': ACTIVATE_FISH,
+            'shim': SHIM
+        }
         bin_dir = join(env_dir, 'bin')
         shim_node = join(bin_dir, "node")
         shim_nodejs = join(bin_dir, "nodejs")
+    if is_CYGWIN:
+        mkdir(bin_dir)
 
     if opt.node == "system":
         files["node"] = SHIM
@@ -803,13 +852,26 @@ def install_activate(env_dir, opt):
         content = content.replace('__SHIM_NODE__', shim_node)
         content = content.replace('__BIN_NAME__', os.path.basename(bin_dir))
         content = content.replace('__MOD_NAME__', mod_dir)
+        if is_CYGWIN:
+            _, cyg_bin_dir = callit(
+                ['cygpath', '-w', os.path.abspath(bin_dir)],
+                show_stdout=False, in_shell=False)
+            content = content.replace('__NPM_CONFIG_PREFIX__', cyg_bin_dir[0])
+        else:
+            content = content.replace('__NPM_CONFIG_PREFIX__',
+                                      '$NODE_VIRTUAL_ENV')
         # if we call in the same environment:
         #   $ nodeenv -p --prebuilt
         #   $ nodeenv -p --node=system
         # we should get `bin/node` not as binary+string.
         # `bin/activate` should be appended if we inside
         # existing python's virtual environment
-        need_append = 0 if name in ('node', 'shim') else opt.python_virtualenv
+        need_append = False
+        if opt.python_virtualenv:
+            disable_prompt = DISABLE_PROMPT.get(name, '')
+            enable_prompt = ENABLE_PROMPT.get(name, '')
+            content = disable_prompt + content + enable_prompt
+            need_append = bool(disable_prompt)
         writefile(file_path, content, append=need_append)
 
     if not os.path.exists(shim_nodejs):
@@ -850,7 +912,7 @@ def create_environment(env_dir, opt):
     # for install
     install_activate(env_dir, opt)
     if node_version_from_opt(opt) < parse_version("0.6.3") or opt.with_npm:
-        instfunc = install_npm_win if is_WIN else install_npm
+        instfunc = install_npm_win if is_WIN or is_CYGWIN else install_npm
         instfunc(env_dir, src_dir, opt)
     if opt.requirements:
         install_packages(env_dir, opt)
@@ -932,7 +994,7 @@ def get_last_stable_node_version():
     """
     Return last stable node.js version
     """
-    response = urlopen('https://%s/dist/latest/' % (src_domain))
+    response = urlopen('https://%s/dist/latest/' % src_domain)
     href_parser = GetsAHrefs()
     href_parser.feed(response.read().decode('UTF-8'))
 
@@ -957,6 +1019,8 @@ def get_env_dir(opt, args):
             res = sys.prefix
         elif hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
             res = sys.prefix
+        elif 'CONDA_PREFIX' in os.environ:
+            res = sys.prefix
         else:
             logger.error('No python virtualenv is available')
             sys.exit(2)
@@ -975,6 +1039,7 @@ def is_installed(name):
     return True
 
 
+# noinspection PyProtectedMember
 def main():
     """
     Entry point
@@ -985,6 +1050,7 @@ def main():
         return
 
     opt, args = parse_args(check=False)
+    # noinspection PyProtectedMember
     Config._load(opt.config_file, opt.verbose)
 
     opt, args = parse_args()
@@ -1013,16 +1079,29 @@ def main():
 # ---------------------------------------------------------
 # Shell scripts content
 
-DISABLE_PROMPT = """
+DISABLE_PROMPT = {
+    'activate': """
 # disable nodeenv's prompt
 # (prompt already changed by original virtualenv's script)
 # https://github.com/ekalinin/nodeenv/issues/26
 NODE_VIRTUAL_ENV_DISABLE_PROMPT=1
-"""
+""",
+    'activate.fish': """
+# disable nodeenv's prompt
+# (prompt already changed by original virtualenv's script)
+# https://github.com/ekalinin/nodeenv/issues/26
+set NODE_VIRTUAL_ENV_DISABLE_PROMPT 1
+""",
+}
 
-ENABLE_PROMPT = """
+ENABLE_PROMPT = {
+    'activate': """
 unset NODE_VIRTUAL_ENV_DISABLE_PROMPT
-"""
+""",
+    'activate.fish': """
+set -e NODE_VIRTUAL_ENV_DISABLE_PROMPT
+""",
+}
 
 SHIM = """#!/usr/bin/env bash
 export NODE_PATH=__NODE_VIRTUAL_ENV__/lib/node_modules
@@ -1191,7 +1270,7 @@ freeze () {
     fi
 }
 
-# unset irrelavent variables
+# unset irrelevant variables
 deactivate_node nondestructive
 
 # find the directory of this script
@@ -1222,22 +1301,22 @@ export NODE_PATH
 
 _OLD_NPM_CONFIG_PREFIX="$NPM_CONFIG_PREFIX"
 _OLD_npm_config_prefix="$npm_config_prefix"
-NPM_CONFIG_PREFIX="$NODE_VIRTUAL_ENV"
-npm_config_prefix="$NODE_VIRTUAL_ENV"
+NPM_CONFIG_PREFIX="__NPM_CONFIG_PREFIX__"
+npm_config_prefix="__NPM_CONFIG_PREFIX__"
 export NPM_CONFIG_PREFIX
 export npm_config_prefix
 
 if [ -z "$NODE_VIRTUAL_ENV_DISABLE_PROMPT" ] ; then
     _OLD_NODE_VIRTUAL_PS1="$PS1"
     if [ "x__NODE_VIRTUAL_PROMPT__" != x ] ; then
-        PS1="__NODE_VIRTUAL_PROMPT__$PS1"
+        PS1="__NODE_VIRTUAL_PROMPT__ $PS1"
     else
     if [ "`basename \"$NODE_VIRTUAL_ENV\"`" = "__" ] ; then
         # special case for Aspen magic directories
         # see http://www.zetadev.com/software/aspen/
         PS1="[`basename \`dirname \"$NODE_VIRTUAL_ENV\"\``] $PS1"
     else
-        PS1="(`basename \"$NODE_VIRTUAL_ENV\"`)$PS1"
+        PS1="(`basename \"$NODE_VIRTUAL_ENV\"`) $PS1"
     fi
     fi
     export PS1
@@ -1251,9 +1330,151 @@ if [ -n "$BASH" -o -n "$ZSH_VERSION" ] ; then
 fi
 """
 
+
+ACTIVATE_FISH = """
+
+# This file must be used with "source bin/activate.fish" *from fish*
+# you cannot run it directly
+
+function deactivate_node -d 'Exit nodeenv and return to normal environment.'
+    # reset old environment variables
+    if test -n "$_OLD_NODE_VIRTUAL_PATH"
+        set -gx PATH $_OLD_NODE_VIRTUAL_PATH
+        set -e _OLD_NODE_VIRTUAL_PATH
+    end
+
+    if test -n "$_OLD_NODE_PATH"
+        set -gx NODE_PATH $_OLD_NODE_PATH
+        set -e _OLD_NODE_PATH
+    else
+        set -e NODE_PATH
+    end
+
+    if test -n "$_OLD_NPM_CONFIG_PREFIX"
+        set -gx NPM_CONFIG_PREFIX $_OLD_NPM_CONFIG_PREFIX
+        set -e _OLD_NPM_CONFIG_PREFIX
+    else
+        set -e NPM_CONFIG_PREFIX
+    end
+
+    if test -n "$_OLD_npm_config_prefix"
+        set -gx npm_config_prefix $_OLD_npm_config_prefix
+        set -e _OLD_npm_config_prefix
+    else
+        set -e npm_config_prefix
+    end
+
+    if test -n "$_OLD_NODE_FISH_PROMPT_OVERRIDE"
+        # Set an empty local `$fish_function_path` to allow the removal of
+        # `fish_prompt` using `functions -e`.
+        set -l fish_function_path
+
+        # Erase virtualenv's `fish_prompt` and restore the original.
+        functions -e fish_prompt
+        functions -c _old_fish_prompt fish_prompt
+        functions -e _old_fish_prompt
+        set -e _OLD_NODE_FISH_PROMPT_OVERRIDE
+    end
+
+    if test (count $argv) = 0 -o "$argv[1]" != "nondestructive"
+        # Self destruct!
+        functions -e deactivate_node
+    end
+end
+
+function freeze -d 'Show a list of installed packages - like `pip freeze`'
+    set -l NPM_VER (npm -v | cut -d '.' -f 1)
+    set -l RE "[a-zA-Z0-9\.\-]+@[0-9]+\.[0-9]+\.[0-9]+([\+\-][a-zA-Z0-9\.\-]+)*"
+
+    if test "$NPM_VER" = "0"
+        set -g NPM_LIST (npm list installed active >/dev/null ^/dev/null | \
+                         cut -d ' ' -f 1 | grep -v npm)
+    else
+        set -l NPM_LS "npm ls -g"
+        if test (count $argv) -gt 0 -a "$argv[1]" = "-l"
+            set NPM_LS "npm ls"
+            set -e argv[1]
+        end
+        set -l NPM_LIST (eval $NPM_LS | grep -E '^.{4}\w{1}' | \
+                                        grep -o -E "$re" | \
+                                        grep -v npm)
+    end
+
+    if test (count $argv) = 0
+        echo $NPM_LIST
+    else
+        echo $NPM_LIST > $argv[1]
+    end
+end
+
+# unset irrelevant variables
+deactivate_node nondestructive
+
+# NODE_VIRTUAL_ENV is the parent of the directory where this script is
+set -gx NODE_VIRTUAL_ENV __NODE_VIRTUAL_ENV__
+
+set -gx _OLD_NODE_VIRTUAL_PATH $PATH
+# The node_modules/.bin path doesn't exists and it will print a warning, and
+# that's why we redirect stderr to /dev/null :)
+set -gx PATH "$NODE_VIRTUAL_ENV/lib/node_modules/.bin" "$NODE_VIRTUAL_ENV/__BIN_NAME__" $PATH ^/dev/null
+
+if set -q NODE_PATH
+    set -gx _OLD_NODE_PATH $NODE_PATH
+    set -gx NODE_PATH "$NODE_VIRTUAL_ENV/__MOD_NAME__" $NODE_PATH
+else
+    set -gx NODE_PATH "$NODE_VIRTUAL_ENV/__MOD_NAME__"
+end
+
+if set -q NPM_CONFIG_PREFIX
+    set -gx _OLD_NPM_CONFIG_PREFIX $NPM_CONFIG_PREFIX
+end
+set -gx NPM_CONFIG_PREFIX "__NPM_CONFIG_PREFIX__"
+
+if set -q npm_config_prefix
+    set -gx _OLD_npm_config_prefix $npm_config_prefix
+end
+set -gx npm_config_prefix "__NPM_CONFIG_PREFIX__"
+
+if test -z "$NODE_VIRTUAL_ENV_DISABLE_PROMPT"
+    # Copy the current `fish_prompt` function as `_old_fish_prompt`.
+    functions -c fish_prompt _old_fish_prompt
+
+    function fish_prompt
+        # Save the current $status, for fish_prompts that display it.
+        set -l old_status $status
+
+        # Prompt override provided?
+        # If not, just prepend the environment name.
+        if test -n ""
+            printf '%s%s' "" (set_color normal)
+        else
+            printf '%s(%s) ' (set_color normal) (basename "$NODE_VIRTUAL_ENV")
+        end
+
+        # Restore the original $status
+        echo "exit $old_status" | source
+        _old_fish_prompt
+    end
+
+    set -gx _OLD_NODE_FISH_PROMPT_OVERRIDE "$NODE_VIRTUAL_ENV"
+end
+"""  # noqa
+
 PREDEACTIVATE_SH = """
 if type -p deactivate_node > /dev/null; then deactivate_node;fi
 """
 
+CYGWIN_NODE = """#!/bin/sh
+
+if [ -r "$1" ]; then
+    SCRIPT_PATH=$(cygpath -w "$1")
+    shift
+    set - $SCRIPT_PATH $@
+    unset SCRIPT_PATH
+fi
+
+exec $(dirname "$0")/node.exe "$@"
+"""
+
 if __name__ == '__main__':
     main()
